/*
Copyright 2004 Cesar Tardaguila & Javier Tardaguila

This file is part of Drops.

Drops is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Srops is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Drops; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import net.designnation.events.EventSource
import net.designnation.data.*


class net.designnation.data.HashMap implements ICollection
{
	// Intial size for disperse table 
	//
	private static var INIT_DT_SIZE: Number= 0x0f;
	
	private var itemCount: Number= 0;
	
	private var disperseTableVal: Array;
	
	private var initSizeVal: Number;
	
	public static var COLL_TAG= "collection";
	
	public function HashMap( orderFunctor: IOrder, initSize: Number )
	{
		//this.className= "net.designnation.data.HashMap";
		
		this.initSizeVal= initSize;
		
		if( this.initSizeVal== undefined )
		{
			this.initSizeVal= INIT_DT_SIZE;
		}
		
		disperseTableVal= new Array( this.initSizeVal );			
	}
	
	public function build( pairs: Array ): HashMap
	{
		var tam: Number= pairs.length;
		
		for( var ix: Number= 0; ix< tam; ix+= 2 )
		{
			setItem( pairs[ ix ], pairs[ ix+ 1 ] );
		}
		
		return this;
	}
	
	public function getItem( key: Object ): Object
	{
		var foundItem: Object= undefined;
		
		if( key!= undefined && key!= undefined )
		{
			var listRow: ListOfPairs= getListInRow( key );
		
			if( listRow!= undefined )
			{
				var searchedItem: Pair= new Pair( key, undefined )
		
				var bFound	: Boolean= listRow.searchAndSet( searchedItem )!= -1;
						
				if( bFound )
				{
					foundItem= searchedItem.second;
				}
			}
		}		
		return foundItem;
	}
	
	public function setItem( key: Object, item: Object ): Boolean
	{		
		if( key!= undefined )
		{
			if( item== undefined )
			{
				item= key;
			}
			
			var listRow: ListOfPairs= getListInRow( key, true );
										
			var searchedItem: Pair= new Pair( key, item )
		
			var bFound	: Boolean= listRow.searchAndSet( searchedItem )!= -1;
			
						
			if( !bFound )
			{
				if( item!= undefined )
				{
					listRow.push( searchedItem );
					this.itemCount++;
				}				
			}		
		}
		return bFound;
	}
	
	public function remove( key: Object ): Boolean
	{              
		var retVal: Boolean= false;
		
		if( key!= undefined && key!= undefined )
		{
			var listRow: ListOfPairs= getListInRow( key );
		
			if( listRow!= undefined )
			{
				var searchedItem: Pair= new Pair( key, undefined )
		
				retVal= listRow.deleteElement( searchedItem );
					
				if( retVal )
				{
					this.itemCount--;
				}
			}
		}		
		return retVal;
	}
	
	public function count(  ): Number
	{
		return this.itemCount;
	}
	
	public function iterator(  ): IIterator
	{
		return new HashMapIteratorImpl( this );
	}
	
	public function isEmpty(  ): Boolean
	{
		return this.itemCount== 0;
	}
	
	public function clear(  )
	{
		this.disperseTableVal	= new Array( this.initSizeVal );
		this.itemCount= 0;
	}
		
	public function set value( pair: Pair )
	{
		setItem( pair.first, pair.second );
	}
			
	public function toString(  ): String
	{      
		var retVal: String= "HashMap("+ this.itemCount+ " elements):";
		
		var it: IIterator= this.iterator(  );

		while( it.hasNext(  ) )
		{	
			var pair: Pair= Pair( it.next(  ) );
			
			retVal+= "["+ pair.first.toString(  )+ ", "+ pair.second.toString(  )+ "]";
			
			if( it.hasNext(  ) )
			{
				retVal+= ", ";
			}
		}
		
		return retVal;
	}			
	
	private function getDisperseTable( ): Array
	{
		return disperseTableVal;
	}
	
	private function getListInRow( key: Object, createList: Boolean ): ListOfPairs
	{
		var strKey	: String= key._name;
		
		if( strKey== undefined )
		{
			strKey= key.id;
			
			if( strKey== undefined )
			{
				strKey= key.toString(  );
			}
		}

		var pos		: Number= ( ord( strKey.charAt( 0 ) )+ ord( strKey.charAt( strKey.length- 1 ) ) )% this.disperseTableVal.length;		
		
		var listRow: ListOfPairs= this.disperseTableVal[ pos ];
		
		if( createList && listRow== undefined )
		{
			listRow= new ListOfPairs(  );
			this.disperseTableVal[ pos ]= listRow;
		}
		
		return listRow;
	}
}